@@ -2,7 +2,6 @@
 -- test setting/clearing of the the execute file attribute works
 --
 skip_if(ostype=="Windows")
-skip_if(string.sub(ostype, 1, 6)=="CYGWIN") -- test -x broken
 
 mtn_setup()
 
@@ -30,7 +29,7 @@ check(indir("checkout", {"test", "-x","foo"}, 0, false, false))
 -- test clone with mtn:execute
 
 copy("test.db", "test-clone.db")
-testURI="file://" .. test.root .. "/test-clone.db?testbranch"
+test_uri="file://" .. url_encode_path(test.root .. "/test-clone.db") .. "?testbranch"
 
-check(nodb_mtn("clone", testURI, "clone"), 0, false, true)
+check(nodb_mtn("clone", test_uri, "clone"), 0, false, true)
 check(indir("clone", {"test", "-x","foo"}, 0, false, false))
