@@ -7,4 +7,4 @@ rename("stdout", "a_v")
 -- MinGW's wc produces "      1" as output.  Arithmetic comparison works, string comparison doesn't
 check(numlines("a_v") == 1)
 -- This is really ^[0-9]+\.[0-9]+$, but m4 is obfuscatory.
-check(qgrep("^[0-9]+\.[0-9]+$", "a_v"))
+check(qgrep("^[0-9]+\\.[0-9]+$", "a_v"))
