@@ -1,6 +1,3 @@
--- Cygwin doesn't do write permissions properly
-skip_if(string.sub(ostype, 1, 6)=="CYGWIN")
-
 skip_if(not existsonpath("chmod"))
 mtn_setup()
 
